/*
 *  ifcfg-password.c - pppd plugin to read PAP password from rc-scriptish
 *                     /etc/sysconfig/interfaces/ifcfg-*
 *
 *  Copyright 1999 Paul Mackerras, Alan Curry.
 *  Copyright 2003 Wojtek Kaniewski.
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version
 *  2 of the License, or (at your option) any later version.
 */

#include <errno.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include "pppd.h"

char pppd_version[] = VERSION;

static char ifcfg_path[PATH_MAX + 1];

static option_t options[] = {
	{ "ifcfg", o_string, ifcfg_path, "Path to interface configuration", OPT_STATIC, NULL, PATH_MAX },
	{ NULL }
};

static int read_password(char *user, char *passwd)
{
	char line[1024];
	int result = 0;
	FILE *f;

	if (!passwd)
		return 1;

	if (!ifcfg_path[0]) {
		warn("Missing 'ifcfg' option -- unable to authenticate\n");
		return -1;
	}

	if (!(f = fopen(ifcfg_path, "r"))) {
		warn("Unable to open %s\n", ifcfg_path);
		return -1;
	}

	while (fgets(line, sizeof(line) - 1, f)) {
		char *p, *q;
		int dq = 0;	/* double-quoted password */

		if (strncmp(line, "PASSWORD=", 9))
			continue;

		if (line[strlen(line) - 1] == '\n')
			line[strlen(line) - 1] = 0;
		if (line[strlen(line) - 1] == '\r')
			line[strlen(line) - 1] = 0;

		p = line + 9;

		switch (*p) {
			case '\'':
				p++;

				for (q = p; *q; q++) {
					if (*q == '\'') {
						*q = 0;
						break;
					}
				}

				strncpy(passwd, p, MAXSECRETLEN - 1);

				break;

			case '\"':
				p++;
				dq = 1;

			default:
				q = passwd;

				for (; *p; p++, q++) {
					switch (*p) {
						case '\\':
							if (*(p + 1)) {
								p++;
								*q = *p;
							}

							break;
							
						case '"':
							*q = (dq) ? 0 : '"';
							break;

						default:
							*q = *p;
					}

					if (!*q)
						break;
					
					if ((int) (q - passwd) > MAXSECRETLEN - 3) {
						warn("Password too long\n");
						q++;
						*q = 0;
					}

				}

				*q = 0;
		}

		passwd[MAXSECRETLEN - 1] = 0;

		result = 1;
	}

	if (!result)
		warn("Password not found in %s\n", ifcfg_path);

	fclose(f);

	return result;
}

void plugin_init(void)
{
	add_options(options);
	pap_passwd_hook = read_password;
}
